local AceGUI = LibStub("AceGUI-3.0")
do 
	local widgetType = "Yata_TotemButtonLayoutEditor"
	local widgetVersion = 1
	local startIcon = 0
	local stopIcon = 0
	local icons = {}
	local function Frame_OnEnter(this)
		local self = this.obj
		self:Fire("OnEnter")
	end

	local function Frame_OnLeave(this)
		local self = this.obj
		self:Fire("OnLeave")		
	end

	local function OnAcquire(self)
	end
	
	local function OnRelease(self)
		self:Hide()
	end
	
	local function Show(self)
		self.main:Show()
	end

	local function Hide(self)
		ClearCursor()
		self.main.frame:ClearAllPoints()
		self.main:Hide()
	end
	local function IconDrag(self, start)		
		if start then
			PickupSpell( self.Button.Totem.SpellID, BOOKTYPE_SPELL );
			startIcon = self
		else 
			ClearCursor()
			stopIcon = self
			if startIcon and stopIcon then
				if math.floor(startIcon.id/10) == math.floor(stopIcon.id/10) then
					local idbackup = stopIcon.id
					stopIcon.id = startIcon.id
					startIcon.id = idbackup
					startIcon.Button:SetAttribute("newposition",math.fmod(startIcon.id,10))
					stopIcon.Button:SetAttribute("newposition",math.fmod(stopIcon.id,10))
				elseif math.fmod(startIcon.id,10)  == math.fmod(stopIcon.id,10) then
					local backup = stopIcon.Button.Totem.Element
					local oldindex = math.floor(startIcon.id/10)
					local newindex = math.floor(stopIcon.id/10)
					Yata.db.profile.BarTotemOrder.ElementsOrder[newindex] = startIcon.Button.Totem.Element
					Yata.db.profile.BarTotemOrder.ElementsOrder[oldindex] = backup
					for m,n in ipairs(icons) do
						local nid = math.floor(n.id/10)
						if nid == oldindex then
							nid = newindex
						elseif nid == newindex then							
							nid = oldindex
						end
						n.id = nid * 10 + math.fmod(n.id,10)
					end
				end
				for m,n in ipairs(icons) do
					n:ClearAllPoints()
					n:SetPoint("BOTTOMLEFT",n:GetParent(),"BOTTOMLEFT",100+(math.floor(n.id/10)-1)*36,100+math.fmod(n.id,10)*36)	
				end
				Yata:SetBarConfiguration(Yata.db.profile.ButtonGap, Yata.db.profile.BarOrientation, Yata.db.profile.VisibleButtons)
			end
			startIcon = nil
			stopIcon = nil			
		end
	end
	
	local function TotemClick(self)
		if not self.Button:GetAttribute("hidden") then
			self.Visibility:SetVertexColor(1.0, 0.0, 0, 0.75)
			self.Visibility:SetText("Hide")
			self.Button:SetAttribute("hidden", true)
		else
			self.Visibility:SetVertexColor(0.0, 1.0, 0, 0.75)
			self.Visibility:SetText("Show")
			self.Button:SetAttribute("hidden", false)
		end
	end
	
	local function CreateTotemIcon(button, id, parent)
		local name = "Option"..button.Totem.Name
		local totemFrame = CreateFrame("Button", name, parent)
		totemFrame.id = id
		totemFrame:RegisterForClicks("LeftButtonUp","RightButtonUp")
		totemFrame.Button = button
		totemFrame:EnableMouse(true)
		totemFrame:RegisterForDrag("LeftButton")
		totemFrame:SetScript("OnClick", function() TotemClick(totemFrame) end)
		totemFrame:SetScript("OnDragStart", function() IconDrag(totemFrame, true) end)
		totemFrame:SetScript("OnReceiveDrag", function() IconDrag(totemFrame, nil) end)
		totemFrame:SetWidth(36)
		totemFrame:SetHeight(36)
		totemFrame:SetFrameLevel(parent:GetFrameLevel() + 2)
		totemFrame.Icon = totemFrame:CreateTexture(name.."Icon")   
		totemFrame.Icon:SetTexture(button.Totem.Texture)			
		totemFrame.Icon:SetAllPoints(totemFrame)
		totemFrame.Visibility = totemFrame:CreateFontString(name.."Visibility", "Overlay", "GameFontNormalSmall")	
		totemFrame.Visibility:SetWidth(36)
		totemFrame.Visibility:SetHeight(10)
		if button:GetAttribute("hidden") then
			totemFrame.Visibility:SetText("Hide")
			totemFrame.Visibility:SetVertexColor(1.0, 0.0, 0, 0.75)
		else
			totemFrame.Visibility:SetText("Show")
			totemFrame.Visibility:SetVertexColor(0.0, 1.0, 0, 0.75)
		end
		totemFrame.Visibility:SetPoint("CENTER",0,-10)			
		return totemFrame
	end		
	
	local function Constructor()
		local self = {}
		self.main = AceGUI:Create("Frame")
		self.type = widgetType
		self.OnRelease = OnRelease
		self.OnAcquire = OnAcquire
		self.main.frame:SetFrameLevel(10)
		self.main.frame:SetScript("OnEnter", Frame_OnEnter)
		self.main.frame:SetScript("OnLeave", Frame_OnLeave)
		self.main.titletext:SetText("Layout Editor")
		self.main.closebutton:HookScript("OnClick",function() Yata:SaveBarConfig() end)
		self.Show = Show
		self.Hide = Hide
		for k,v in ipairs(Yata.db.profile.BarTotemOrder.ElementsOrder) do
			for m,n in ipairs(Yata.TotemButtons[v].Buttons) do
				local totem = n.Totem
				local id = k*10+n:GetAttribute("position")
				local tIcon = CreateTotemIcon(n,id,self.main.frame)
				tIcon:SetPoint("BOTTOMLEFT",self.main.frame,"BOTTOMLEFT",100+(math.floor(id/10)-1)*36,100+math.fmod(id,10)*36)		
				table.insert(icons,tIcon)
			end
		end
		return self
	end

			
	AceGUI:RegisterWidgetType(widgetType, Constructor, widgetVersion)
end